<?php

namespace Drupal\dkan_json_form_tweak;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\StringTranslation\StringTranslationTrait;

class FormBuilder extends \Drupal\json_form_widget\FormBuilder {

  use StringTranslationTrait;

  public function getJsonForm($data, $form_state = NULL) {
    $form = parent::getJsonForm($data, $form_state);
    if ($this->settingEnabled($form_state, 'navigation')) {
      $items = [];
      // $form['data']['data'] means it is a sub schema, not dataset
      $form_part = $form['data']['data'] ?? $form;
      foreach (Element::children($form_part) as $item_key) {
        if ((!isset($form_part[$item_key]['#access']) || $form_part[$item_key]['#access'] === TRUE) && isset($form_part[$item_key]['#title'])) {
          $items[] = ['#markup' => $form_part[$item_key]['#title']];
        }
        if ((!isset($form_part[$item_key][$item_key]['#access']) || $form_part[$item_key][$item_key]['#access'] === TRUE) && isset($form_part[$item_key][$item_key]['#title'])) {
          $items[] = ['#markup' => $form_part[$item_key][$item_key]['#title']];
        }
      }
      $navigation = [
        '#theme' => 'dkan_json_form_navigation',
        '#label' => ['#markup' => $this->t('Go to property:')],
        '#items' => $items,
        '#weight' => -1
      ];
      if (isset($form['data']['data'])) {
        $form['data']['data']['#attributes']['data-json-form-navigation'] = 'true';
        $form['data']['data']['navigation'] = $navigation;
      }
      else {
        $form['navigation'] = $navigation;
      }
      $form['#attached']['library'][] = 'dkan_json_form_tweak/dkan_json_form_navigation';
    }
    if ($this->settingEnabled($form_state, 'close_details') || $this->settingEnabled($form_state, 'remove_multivalue')) {
      if (isset($form['data']['data'])) {
        foreach (Element::children($form['data']['data']) as $item_key) {
          if (isset($form['data']['data'][$item_key][$item_key]) && is_array($form['data']['data'][$item_key][$item_key]) && count($form['data']['data'][$item_key][$item_key]) >= 1) {
            $first_item = reset($form['data']['data'][$item_key][$item_key]);
            // Put simple array elements in container to show remove checkbox
            // remove that container later in ValueHandler::flattenArraysInArrays
            if ($this->settingEnabled($form_state, 'remove_multivalue')) {
              if (count($form['data']['data'][$item_key][$item_key]) > 1 && isset($first_item['#type'])) {
                foreach ($form['data']['data'][$item_key][$item_key] as &$value) {
                  $container = [
                    '#type' => 'container'
                  ];
                  $container['value'] = $value;
                  $container['remove_item'] = [
                    '#type' => 'checkbox',
                    '#title' => $this->t('Remove'),
                    '#description' => $this->t('Remove this item permanently.'),
                  ];
                  $value = $container;
                }
              }
            }
            if (isset($first_item[$item_key]['#type']) && $first_item[$item_key]['#type'] == 'details') {
              if ($this->settingEnabled($form_state, 'remove_multivalue')) {
                foreach ($form['data']['data'][$item_key][$item_key] as &$value) {
                  $value[$item_key]['remove_item'] = [
                    '#type' => 'checkbox',
                    '#title' => $this->t('Remove'),
                    '#description' => $this->t('Remove this item permanently.'),
                  ];
                }
              }
              if ($this->settingEnabled($form_state, 'close_details')) {
                $form['data']['data'][$item_key][$item_key]['close_button'] = [
                  '#theme' => 'dkan_json_form_close_button',
                  '#label' => $this->t('Close'),
                  '#status' => 'opened',
                  '#weight' => -1,
                ];
              }
            }
          }
        }
      }
      else {
        foreach ($form as $item_key => &$item) {
          if (isset($item[$item_key]) && is_array($item[$item_key]) && count($item[$item_key]) >= 1) {
            $first_item = reset($item[$item_key]);
            // Put simple array elements in container to show remove checkbox
            // remove that container later in ValueHandler::flattenArraysInArrays
            if ($this->settingEnabled($form_state, 'remove_multivalue')) {
              if (count($item[$item_key]) > 1 && isset($first_item['#type'])) {
                foreach ($item[$item_key] as &$value) {
                  $container = [
                    '#type' => 'container'
                  ];
                  $container['value'] = $value;
                  $container['remove_item'] = [
                    '#type' => 'checkbox',
                    '#title' => $this->t('Remove'),
                    '#description' => $this->t('Remove this item permanently.'),
                  ];
                  $value = $container;
                }
              }
            }
            if (isset($first_item[$item_key]['#type']) && $first_item[$item_key]['#type'] == 'details') {
              if ($this->settingEnabled($form_state, 'close_details')) {
                $item['close_button'] = [
                  '#theme' => 'dkan_json_form_close_button',
                  '#label' => $this->t('Close'),
                  '#status' => 'opened',
                  '#weight' => -1,
                ];
              }
              if ($this->settingEnabled($form_state, 'remove_multivalue')) {
                foreach ($item[$item_key] as &$value) {
                  $value[$item_key]['remove_item'] = [
                    '#type' => 'checkbox',
                    '#title' => $this->t('Remove'),
                    '#description' => $this->t('Remove this item permanently.'),
                  ];
                }
              }
            }
          }
        }
      }
      $form['#attached']['library'][] = 'dkan_json_form_tweak/dkan_json_form_close_details';
      $form['#attached']['drupalSettings']['dkan_json_form_tweak']['close_label'] = $this->t('Close');
      $form['#attached']['drupalSettings']['dkan_json_form_tweak']['open_label'] = $this->t('Open', [], ['context' => 'opening']);
    }
    return $form;
  }

  protected function settingEnabled(?FormStateInterface $form_state, $setting) {
    $form_display = $form_state ? $form_state->getStorage()['form_display'] : null;
    if ($form_display instanceof EntityFormDisplay) {
      return $form_display->getThirdPartySetting('dkan_json_form_tweak', $setting);
    }
    return FALSE;
  }

}
